mgraphics.init();
mgraphics.relative_coords = 0;
mgraphics.autofill = 0;

var kDMXAddressCount = 512;
var kDMXValueCount = 256;

var gridMonitorStyles = {
  targetCell: {
    width: 35,
    height: 40,
    margin: 5,
  },
  address: {
    positionXRatio: 0.5,
    get positionX() {
      return Math.floor(
        gridMonitorStyles.targetCell.width *
          gridMonitorStyles.address.positionXRatio
      );
    },
    positionYRatio: 0.325,
    get positionY() {
      return Math.floor(
        gridMonitorStyles.targetCell.height *
          gridMonitorStyles.address.positionYRatio
      );
    },
    color: [0.46, 0.46, 0.46],
    fontSize: 11.0,
    fontFamily: "Lato",
  },
  dividerLine: {
    paddingLeftRight: 4,
    get positionX() {
      return gridMonitorStyles.dividerLine.paddingLeftRight;
    },
    positionYRatio: 0.45,
    get positionY() {
      return Math.floor(
        gridMonitorStyles.targetCell.height *
          gridMonitorStyles.dividerLine.positionYRatio
      );
    },
    get width() {
      return (
        gridMonitorStyles.targetCell.width -
        gridMonitorStyles.dividerLine.paddingLeftRight * 2
      );
    },
    height: 1,
    backgroundColor: [0.38, 0.38, 0.38],
  },
  value: {
    positionXRatio: 0.5,
    get positionX() {
      return Math.floor(
        gridMonitorStyles.targetCell.width *
          gridMonitorStyles.value.positionXRatio
      );
    },
    positionYRatio: 0.85,
    get positionY() {
      return Math.floor(
        gridMonitorStyles.targetCell.height *
          gridMonitorStyles.value.positionYRatio
      );
    },
    color: [1.0, 1.0, 1.0],
    fontSize: 11.0,
    fontFamily: "Lato",
  },
  valueSlider: {
    backgroundColor: [0.173, 0.275, 0.49],
  }
};

function getColumnCount(windowWidth) {
  return Math.floor(
    windowWidth /
      (gridMonitorStyles.targetCell.width + gridMonitorStyles.targetCell.margin)
  );
}
getColumnCount.local = 1;

function getDigitsOffsetX(number) {
  var digitCount = number.toString().length;
  if (digitCount == 1) {
    return -3;
  } else if (digitCount == 2) {
    return -5;
  } else if (digitCount >= 3) {
    return -8;
  }
}
getDigitsOffsetX.local = 1;

function drawAddresses(columnCount) {
  mgraphics.set_font_size(gridMonitorStyles.address.fontSize);
  mgraphics.select_font_face(gridMonitorStyles.address.fontFamily);

  for (var address = 0; address < kDMXAddressCount; address++) {
    // Draw address digit
    var digitsOffsetX = getDigitsOffsetX(address + 1);
    var digitsPositionX =
      (gridMonitorStyles.targetCell.width +
        gridMonitorStyles.targetCell.margin) *
        (address % columnCount) +
      gridMonitorStyles.address.positionX +
      digitsOffsetX;
    var digitsPositionY =
      (gridMonitorStyles.targetCell.height +
        gridMonitorStyles.targetCell.margin) *
        Math.floor(address / columnCount) +
      gridMonitorStyles.address.positionY;

    mgraphics.set_source_rgb(gridMonitorStyles.address.color);
    mgraphics.move_to(digitsPositionX, digitsPositionY);
    mgraphics.show_text("" + (address + 1) + "");
    mgraphics.fill();

    // Draw a divider line between address and value digits
    var dividerPositionX =
      (gridMonitorStyles.targetCell.width +
        gridMonitorStyles.targetCell.margin) *
        (address % columnCount) +
      gridMonitorStyles.dividerLine.positionX;
    var dividerPositionY =
      (gridMonitorStyles.targetCell.height +
        gridMonitorStyles.targetCell.margin) *
        Math.floor(address / columnCount) +
      gridMonitorStyles.dividerLine.positionY;

    mgraphics.set_source_rgb(gridMonitorStyles.dividerLine.backgroundColor);
    mgraphics.rectangle(
      dividerPositionX,
      dividerPositionY,
      gridMonitorStyles.dividerLine.width,
      gridMonitorStyles.dividerLine.height
    );
    mgraphics.fill();
  }
}
drawAddresses.local = 1;

var monitorData = [];
function drawValues(monitorData, columnCount) {
  mgraphics.set_font_size(gridMonitorStyles.value.fontSize);
  mgraphics.select_font_face(gridMonitorStyles.value.fontFamily);

  for (var address = 0; address < monitorData.length; address++) {
    var value = monitorData[address];

    // Draw value slider
    var sliderPositionX =
      (gridMonitorStyles.targetCell.width +
        gridMonitorStyles.targetCell.margin) *
      (address % columnCount);
    var sliderPositionY =
      (gridMonitorStyles.targetCell.height +
        gridMonitorStyles.targetCell.margin) *
        Math.floor(address / columnCount) +
      gridMonitorStyles.targetCell.height;

    mgraphics.set_source_rgb(gridMonitorStyles.valueSlider.backgroundColor);
    mgraphics.rectangle(
      sliderPositionX,
      sliderPositionY,
      gridMonitorStyles.targetCell.width,
      gridMonitorStyles.targetCell.height * (value / (kDMXValueCount - 1)) * -1
    );
    mgraphics.fill();

    // Draw value digits
    var digitsOffsetX = getDigitsOffsetX(value);
    var digitsPositionX =
      (gridMonitorStyles.targetCell.width +
        gridMonitorStyles.targetCell.margin) *
        (address % columnCount) +
      gridMonitorStyles.value.positionX +
      digitsOffsetX;
    var digitsPositionY =
      (gridMonitorStyles.targetCell.height +
        gridMonitorStyles.targetCell.margin) *
        Math.floor(address / columnCount) +
      gridMonitorStyles.value.positionY;

    mgraphics.set_source_rgb(gridMonitorStyles.value.color);
    mgraphics.move_to(digitsPositionX, digitsPositionY);
    mgraphics.show_text("" + value + "");
    mgraphics.fill();
  }
}
drawValues.local = 1;

function draw() {
  mgraphics.redraw();
}
draw.local = 1;

function paint() {
  var width = this.box.rect[2] - this.box.rect[0];

  // Draw DMX values and addresses
  var columnCount = getColumnCount(width + gridMonitorStyles.targetCell.margin);
  drawValues(monitorData, columnCount);
  drawAddresses(columnCount);
}

function set() {
  monitorData = arguments;

  draw();
}

function clear() {
  monitorData = [];

  draw();
}