var VERSION = 1;
var PATCH_DICT_ID = jsarguments[1];
var PROFILES_DICT_ID = jsarguments[2];

var patchDict = new Dict(PATCH_DICT_ID);
var patchObj = {};

var profilesDict = new Dict(PROFILES_DICT_ID);
var profilesObj = {};

function loadPatchDict() {
  patchObj = JSON.parse(patchDict.stringify());
}

function loadProfilesDict() {
  profilesObj = JSON.parse(profilesDict.stringify());
}

function updateDict(targetDict, targetObj) {
  targetDict.parse(JSON.stringify(targetObj));
}

function updatedFixtureNotification(parentFixtureIndex) {
  outlet(0, PATCH_DICT_ID, "fixtureUpdate", parentFixtureIndex, -1);
}

function updatedFixtureListNotification() {
  outlet(0, PATCH_DICT_ID, "fixtureListUpdate");
}

function updatedTagListNotification(parentFixtureIndex) {
  outlet(0, PATCH_DICT_ID, "tagListUpdate", parentFixtureIndex);
}

function removedFixtureNotification() {
  outlet(0, PATCH_DICT_ID, "removedFixture");
}

function initPatch() {
  loadPatchDict();

  if (!patchObj.hasOwnProperty("version")) {
    patchObj.version = VERSION;
  }
  if (!patchObj.hasOwnProperty("fixtures")) {
    patchObj.fixtures = [];
  }

  updateDict(patchDict, patchObj);
}

function newPatch() {
  loadPatchDict();

  patchObj.version = VERSION;
  patchObj.fixtures = [];

  updateDict(patchDict, patchObj);
  updatedFixtureListNotification();
}

function ProfileWithType(type, profiles) {
  for (var profileName in profiles) {
    if (profileName === type) {
      return profiles[profileName];
    }
  }
}

function WithTagOffset(tags, offset) {
  var tagList = [];

  for (var i = 0; i < tags.length; i++) {
    tagList.push({
      name: tags[i].name,
      index: tags[i].index + offset
    });
  }

  return tagList;
}

function CreateTags(names, offset) {
  var tagList = [];

  for (var i = 0; i < names.length; i++) {
    tagList.push({
      name: names[i],
      index: 60 + offset,
    });
  }

  return tagList;
}

function CreateFixture(fixture) {
  return {
    name: fixture.name,
    type: fixture.type,
    channel: fixture.channel,
    universe: fixture.universe,
    tags: fixture.tags,
    ranges: [],
    testOn: false,
    children: []
  };
}

function ChildrenForFixtures(parent, profiles) {
  var fixtureList = [];

  var parentProfile = ProfileWithType(parent.type, profiles);
  var children = parentProfile.children || [];

  for (var i = 0; i < children.length; i++) {
    var profile = ProfileWithType(children[i].profile, profiles);
    var type = profile ? children[i].profile : "";
    var tags = WithTagOffset(parent.tags, i);
    var ranges = parent.ranges;
    var universe = parent.universe;
    var channel = parent.channel + (children[i].address - 1);
    var name = children[i].name;
    var fixture = CreateFixture({
      name: name,
      type: type,
      universe: universe,
      channel: channel,
      tags: tags,
      ranges: ranges
    });

    fixtureList.push(fixture);
  }

  return fixtureList;
}

function CreateFixtures(amount, data, profiles) {
  var fixtureList = [];

  var profile = ProfileWithType(data.type, profiles);
  if (profile === undefined) {
    return fixtureList;
  }

  var tagOffset = 0;
  for (var i = 0; i < amount; i++) {
    var channel = data.address + i * profile.addresses;
    var tags = CreateTags(data.tagNames, tagOffset);
    var fixture = CreateFixture({
      name: data.name,
      type: data.type,
      channel: channel,
      universe: data.universe,
      tags: tags,
    });

    tagOffset += profile.children.length || 1;

    fixture.children = ChildrenForFixtures(fixture, profiles);

    fixtureList.push(fixture);
  }

  return fixtureList;
}

function addFixtures(
  enteredName,
  selectedProfile,
  amountOfFixtures,
  startUniverse,
  startAddress,
  tagNames
) {
  var tags = [];
  if (arguments.length > 6) {
    for (var i = 0; i < arguments.length - 5; i++) {
      tags.push(arguments[i + 5]);
    }
  } else if (arguments.length == 6) {
    tags.push(arguments[5]);
  }

  var data = {
    name: enteredName,
    type: selectedProfile,
    universe: startUniverse,
    address: startAddress,
    tagNames: tags
  };

  loadProfilesDict();
  var newFixtures = CreateFixtures(
    amountOfFixtures,
    data,
    profilesObj
  );

  initPatch();

  patchObj.fixtures = patchObj.fixtures.concat(newFixtures);

  updateDict(patchDict, patchObj);
  updatedFixtureListNotification();
}

function removeAllFixtures() {
  patchObj.fixtures = [];

  updateDict(patchDict, patchObj);
  updatedFixtureListNotification();
}

function removeFixture(parentFixtureIndex) {
  loadPatchDict();
  var patch = patchObj.fixtures;
  if (!patch) return;

  var parent = patch[parentFixtureIndex];
  if (!parent) return;

  patch.splice(parentFixtureIndex, 1);

  updateDict(patchDict, patchObj);
  removedFixtureNotification();
}

function setFixtureName(parentFixtureIndex, newFixtureName) {
  loadPatchDict();
  var patch = patchObj.fixtures;
  if (!patch) return;

  var parent = patch[parentFixtureIndex];
  if (!parent) return;

  parent["name"] = newFixtureName;

  updateDict(patchDict, patchObj);
  updatedFixtureNotification(parentFixtureIndex);
}

function setFixtureProfile(parentFixtureIndex, newProfile) {
  loadPatchDict();
  var patch = patchObj.fixtures;
  if (!patch) return;

  var parent = patch[parentFixtureIndex];
  if (!parent) return;

  loadProfilesDict();
  var profiles = profilesObj;

  var profile = ProfileWithType(newProfile, profiles);
  if (profile === undefined) {
    return;
  }

  parent.type = newProfile;
  parent.children = ChildrenForFixtures(parent, profiles);

  updateDict(patchDict, patchObj);
  updatedFixtureListNotification();
}

function setFixtureUniverse(parentFixtureIndex, newUniverse) {
  loadPatchDict();
  var patch = patchObj.fixtures;
  if (!patch) return;

  var parent = patch[parentFixtureIndex];
  if (!parent) return;

  parent.universe = newUniverse;
  
  for (var i = 0; i < parent.children.length; i++) {
    parent.children[i].universe = newUniverse;
  }

  updateDict(patchDict, patchObj);
  updatedFixtureNotification(parentFixtureIndex);
}

function setFixtureStartAddress(parentFixtureIndex, newStartAddress) {
  loadPatchDict();
  var patch = patchObj.fixtures;
  if (!patch) return;

  var parent = patch[parentFixtureIndex];
  if (!parent) return;

  parent.channel = newStartAddress;
  
  for (var i = 0; i < parent.children.length; i++) {
    parent.children[i].channel = newStartAddress + i; 
  }

  updateDict(patchDict, patchObj);
  updatedFixtureNotification(parentFixtureIndex);
}

function deleteTagFromFixture(fixture, tagName) {
  var tags = fixture.tags;

  var existing = -1;
  for (var i = 0; i < tags.length; i++) {
    if (tags[i]["name"] === tagName) {
      existing = i;
      break;
    }
  }

  if (existing >= 0) {
    tags.splice(existing, 1);
    return fixture;
  }

  return fixture;
}

function addTagToFixture(fixture, tag, offset) {
  deleteTagFromFixture(fixture, tag["name"]);

  var tags = fixture.tags;
  var newTag = {
    name: tag["name"],
    index: tag["index"] + offset
  };

  tags.push(newTag);
}

function addTag(parentFixtureIndex) {
  loadPatchDict();
  var patch = patchObj.fixtures;
  if (!patch) return;

  var parent = patch[parentFixtureIndex];
  if (!parent) return;

  var newTag = {
    name: "",
    index: 60
  };

  addTagToFixture(parent, newTag, 0);

  var children = parent.children;
  for (var i = 0; i < children.length; i++) {
    var child = children[i];
    addTagToFixture(child, newTag, i);
  }

  updateDict(patchDict, patchObj);
  updatedTagListNotification(parentFixtureIndex);
}

function setTagName(parentFixtureIndex, currentTagName, newTagName) {
  loadPatchDict();
  var patch = patchObj.fixtures;
  if (!patch) return;

  var parent = patch[parentFixtureIndex];
  if (!parent) return;

  var tags = parent.tags;
  var currentTagIndex;
  for (var i = 0; i < tags.length; i++) {
    if (tags[i]["name"] === currentTagName) {
      currentTagIndex = tags[i]["index"];
      break;
    }
  }

  deleteTagFromFixture(parent, currentTagName);

  var newTag = {
    name: newTagName,
    index: currentTagIndex
  };

  addTagToFixture(parent, newTag, 0);

  var children = parent.children;
  for (var i = 0; i < children.length; i++) {
    var child = children[i];
    deleteTagFromFixture(child, currentTagName);
    addTagToFixture(child, newTag, i);
  }

  updateDict(patchDict, patchObj);
  updatedTagListNotification(parentFixtureIndex);
}

function removeTag(parentFixtureIndex, tagName) {
  loadPatchDict();
  var patch = patchObj.fixtures;
  if (!patch) return;

  var parent = patch[parentFixtureIndex];
  if (!parent) return;

  deleteTagFromFixture(parent, tagName);

  var children = parent.children;
  for (var i = 0; i < children.length; i++) {
    var child = children[i];
    deleteTagFromFixture(child, tagName);
  }

  updateDict(patchDict, patchObj);
  updatedTagListNotification(parentFixtureIndex);
}

function setFixtureTestOn(parentFixtureIndex, testOn) {
  loadPatchDict();
  var patch = patchObj.fixtures;
  if (!patch) return;

  var parent = patch[parentFixtureIndex];
  if (!parent) return;

  if (testOn === 1) {
    testOn = true;
  } else {
    testOn = false;
  }
  parent.testOn = testOn;
  
  for (var i = 0; i < parent.children.length; i++) {
    parent.children[i].testOn = testOn; 
  }

  updateDict(patchDict, patchObj);
  updatedFixtureNotification(parentFixtureIndex);
}

function setFixturePositionUp(parentFixtureIndex) {
  loadPatchDict();
  var patch = patchObj.fixtures;
  if (!patch) return;

  if (parentFixtureIndex >= (patch.length - 1)) return;

  var parent = patch[parentFixtureIndex];
  if (!parent) return;

  arraymove(patch, parentFixtureIndex, parentFixtureIndex + 1);

  updateDict(patchDict, patchObj);
  updatedFixtureListNotification();
}

function setFixturePositionDown(parentFixtureIndex) {
  if (parentFixtureIndex <= 0) return;

  loadPatchDict();
  var patch = patchObj.fixtures;
  if (!patch) return;

  var parent = patch[parentFixtureIndex];
  if (!parent) return;

  arraymove(patchObj.fixtures, parentFixtureIndex, parentFixtureIndex - 1);

  updateDict(patchDict, patchObj);
  updatedFixtureListNotification();
}

function arraymove(arr, fromIndex, toIndex) {
  var element = arr[fromIndex];
  arr.splice(fromIndex, 1);
  arr.splice(toIndex, 0, element);
}